from dataclasses import dataclass
from typing import Iterable

from cml.ports.source_adapters import Adapter
from cml.ports.ml_adapter import MachineLearningModel
from cml.domain.reconstruction import Reconstructor
from cml.domain.construction import Constructor
from cml.domain.complexity_reduction import FeatureSelector
from cml.domain.deconstruction import Deconstructor
from cml.domain.knowledge import KnowledgeDatabase
from cml.shared.settings import (
    GeneralSettings,
    PreprocessingSettings,
    BlockProcessingSettings,
    DeconstructionSettings,
    ConstructionSettings,
    FeatureSelectionSettings,
    ReconstructionSettings)


@dataclass
class PreprocessRequest:
    source_adapter: object
    general_settings: GeneralSettings
    preprocessing_settings: PreprocessingSettings
    block_processing_settings: BlockProcessingSettings
    density_estimator_interface: object
    relative_extrema_interface: object


@dataclass
class ConstructionRequest:
    settings: ConstructionSettings
    algorithms: Iterable
    interface: type
    mode: str


@dataclass
class FeatureSelectionRequest:
    filter_method: MachineLearningModel
    embedded_method: MachineLearningModel
    settings: FeatureSelectionSettings


@dataclass
class ReconstructionRequest:
    settings: ReconstructionSettings
    algorithms: Iterable
    interface: type
    mode: str


@dataclass
class DeconstructionRequest:
    mode: str
    source_adapter: Adapter
    settings: DeconstructionSettings
    knowledge_database: KnowledgeDatabase = None


@dataclass
class KnowledgeSearchRequest:
    constructor: Constructor
    feature_selector: FeatureSelector
    reconstructor: Reconstructor
    deconstructor: Deconstructor
    stdout: bool
    parallel: bool
